package com.nisum.offertransactionservice.controller;

import com.nisum.offertransactionservice.model.*;
import com.nisum.offertransactionservice.service.EndOfSaleService;
import com.nisum.offertransactionservice.service.EndOfTransactionService;
import com.nisum.offertransactionservice.service.OfferCallingPEService;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@RunWith(MockitoJUnitRunner.class)
@Ignore
public class OfferControllerTest {

    @Mock
    private OfferCallingPEService offerCallingPEService;

    @Mock
    private EndOfTransactionService endOfTransactionService;

    @Mock
    private EndOfSaleService endOfSaleService;

    @Test
    public void getOfferTransactionResponseTest() {
        OfferTransactionRequest offerTransactionRequest = getOfferTransactionRequest();
        Mockito.when(offerCallingPEService.getDiscountedItemList(offerTransactionRequest)).thenReturn(retOfferTransRes());
        OfferTransactionResponse offerTransactionResponse = offerCallingPEService.getDiscountedItemList(offerTransactionRequest);
        Assert.assertNotNull(offerTransactionResponse);
    }

    @Test
    public void endOfSaleTest() throws IOException {
        Mockito.when(endOfSaleService.putEndofSaleResInDb(getEndOfSaleReq())).thenReturn(UUID.randomUUID());
        UUID uuid = endOfSaleService.putEndofSaleResInDb(getEndOfSaleReq());
        Assert.assertNotNull(uuid);
    }

    @Test
    public void endOfTransactionTest() {
        Mockito.when(endOfTransactionService.endOfTransaction(UUID.randomUUID().toString())).thenReturn("End Of Transaction note from Store Producer");
        String res = endOfTransactionService.endOfTransaction(UUID.randomUUID().toString());
        Assert.assertEquals(res, "End Of Transaction note from Store Producer");
    }

    private EndOfSaleReq getEndOfSaleReq() {
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq();
        endOfSaleReq.setOfferTransactionResponse(retOfferTransRes());
        endOfSaleReq.setOffers(returnOffers());
        return endOfSaleReq;
    }

    public List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup("1", "12", "234", "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2", "13", "234", "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }

    private OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05, "cmsCoupons");
        Item item2 = new Item("Thumpsup", "11", 12.32, "cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        return offerTransactionResponse;
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05, "cmsCoupons");
        Item item2 = new Item("Thumpsup", "11", 14.05, "cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionRequest.setItemList(itemList);
        return offerTransactionRequest;
    }

}
